@@ -56,7 +56,7 @@ from ._datestrings import DateStrings
 # Top-level module functions
 #
 #-------------------------------------------------------------------------
-_max_days  = [ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ]
+_max_days = [ 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ]
 _leap_days = [ 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 ]
 
 def gregorian_valid(date_tuple):
@@ -193,7 +193,7 @@ class DateParser:
     converted, the text string is assigned.
     """
 
-    _dhformat_parse = re.compile(".*%(\S).*%(\S).*%(\S).*")
+    _dhformat_parse = re.compile(r".*%(\S).*%(\S).*%(\S).*")
 
     # RFC-2822 only uses capitalized English abbreviated names, no locales.
     _rfc_days = ('Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat')
@@ -400,8 +400,8 @@ class DateParser:
         _ = self._locale.translation.gettext
         self.__init_prefix_tables()
 
-        self._rfc_mon_str  = '(' + '|'.join(list(self._rfc_mons_to_int.keys())) + ')'
-        self._rfc_day_str  = '(' + '|'.join(self._rfc_days) + ')'
+        self._rfc_mon_str = '(' + '|'.join(list(self._rfc_mons_to_int.keys())) + ')'
+        self._rfc_day_str = '(' + '|'.join(self._rfc_days) + ')'
 
         self._bce_str = self.re_longest_first(self.bce)
         self._qual_str = self.re_longest_first(list(self.quality_to_int.keys()))
@@ -409,77 +409,83 @@ class DateParser:
         self._mod_after_str = self.re_longest_first(
             list(self.modifier_after_to_int.keys()))
 
-        self._mon_str  = self.re_longest_first(list(self.month_to_int.keys()))
+        self._mon_str = self.re_longest_first(list(self.month_to_int.keys()))
         self._jmon_str = self.re_longest_first(list(self.hebrew_to_int.keys()))
         self._fmon_str = self.re_longest_first(list(self.french_to_int.keys()))
         self._pmon_str = self.re_longest_first(list(self.persian_to_int.keys()))
         self._imon_str = self.re_longest_first(list(self.islamic_to_int.keys()))
         self._smon_str = self.re_longest_first(list(self.swedish_to_int.keys()))
-        self._cal_str  = self.re_longest_first(list(self.calendar_to_int.keys()))
-        self._ny_str   = self.re_longest_first(list(self.newyear_to_int.keys()))
+        self._cal_str = self.re_longest_first(list(self.calendar_to_int.keys()))
+        self._ny_str = self.re_longest_first(list(self.newyear_to_int.keys()))
 
         self._today_str = self.re_longest_first(self.today + [_("today"),])
 
         # bce, calendar type and quality may be either at the end or at
         # the beginning of the given date string, therefore they will
         # be parsed from the middle and will be in match.group(2).
-        self._bce_re    = re.compile("(.*)\s+%s( ?.*)" % self._bce_str)
+        self._bce_re = re.compile(r"(.*)\s+%s( ?.*)" % self._bce_str)
 
-        self._cal       = re.compile("(.*)\s+\(%s\)( ?.*)" % self._cal_str,
-                                     re.IGNORECASE)
-        self._calny     = re.compile("(.*)\s+\(%s,\s*%s\)( ?.*)" % (self._cal_str,
-                                                                    self._ny_str),
-                                     re.IGNORECASE)
-        self._calny_iso = re.compile("(.*)\s+\(%s,\s*(\d{1,2}-\d{1,2})\)( ?.*)" % self._cal_str,
-                                     re.IGNORECASE)
+        self._cal = re.compile(r"(.*)\s+\(%s\)( ?.*)" % self._cal_str,
+                               re.IGNORECASE)
+        self._calny = re.compile(r"(.*)\s+\(%s,\s*%s\)( ?.*)" %
+                                 (self._cal_str, self._ny_str), re.IGNORECASE)
+        self._calny_iso = re.compile(
+            r"(.*)\s+\(%s,\s*(\d{1,2}-\d{1,2})\)( ?.*)" % self._cal_str,
+            re.IGNORECASE)
 
-        self._ny        = re.compile("(.*)\s+\(%s\)( ?.*)" % self._ny_str,
-                                     re.IGNORECASE)
-        self._ny_iso    = re.compile("(.*)\s+\((\d{1,2}-\d{1,2})\)( ?.*)")
+        self._ny = re.compile(r"(.*)\s+\(%s\)( ?.*)" % self._ny_str,
+                              re.IGNORECASE)
+        self._ny_iso = re.compile(r"(.*)\s+\((\d{1,2}-\d{1,2})\)( ?.*)")
 
-        self._qual     = re.compile("(.* ?)%s\s+(.+)" % self._qual_str,
-                                    re.IGNORECASE)
+        self._qual = re.compile(r"(.* ?)%s\s+(.+)" % self._qual_str,
+                                re.IGNORECASE)
 
-        self._span     = re.compile("(from)\s+(?P<start>.+)\s+to\s+(?P<stop>.+)",
+        self._span = re.compile(r"(from)\s+(?P<start>.+)\s+to\s+(?P<stop>.+)",
+                                re.IGNORECASE)
+        self._range = re.compile(
+            r"(bet|bet.|between)\s+(?P<start>.+)\s+and\s+(?P<stop>.+)",
+            re.IGNORECASE)
+        self._modifier = re.compile(r'%s\s+(.*)' % self._mod_str,
                                     re.IGNORECASE)
-        self._range    = re.compile("(bet|bet.|between)\s+(?P<start>.+)\s+and\s+(?P<stop>.+)",
-                                    re.IGNORECASE)
-        self._modifier = re.compile('%s\s+(.*)' % self._mod_str,
-                                    re.IGNORECASE)
-        self._modifier_after = re.compile('(.*)\s+%s' % self._mod_after_str,
+        self._modifier_after = re.compile(r'(.*)\s+%s' % self._mod_after_str,
                                           re.IGNORECASE)
-        self._abt2     = re.compile('<(.*)>', re.IGNORECASE)
-        self._text     = re.compile('%s\.?(\s+\d+)?\s*,?\s+((\d+)(/\d+)?)?\s*$'
-                                        % self._mon_str, re.IGNORECASE)
+        self._abt2 = re.compile('<(.*)>', re.IGNORECASE)
+        self._text = re.compile(r'%s\.?(\s+\d+)?\s*,?\s+((\d+)(/\d+)?)?\s*$'
+                                % self._mon_str, re.IGNORECASE)
         # this next RE has the (possibly-slashed) year at the string's end
-        self._text2    = re.compile('(\d+)?\s+?%s\.?\s*((\d+)(/\d+)?)?\s*$' % self._mon_str,
-                                    re.IGNORECASE)
-        self._jtext    = re.compile('%s\s+(\d+)?\s*,?\s*((\d+)(/\d+)?)?\s*$' % self._jmon_str,
-                                    re.IGNORECASE)
-        self._jtext2   = re.compile('(\d+)?\s+?%s\s*((\d+)(/\d+)?)?\s*$' % self._jmon_str,
-                                    re.IGNORECASE)
-        self._ftext    = re.compile('%s\s+(\d+)?\s*,?\s*((\d+)(/\d+)?)?\s*$' % self._fmon_str,
-                                    re.IGNORECASE)
-        self._ftext2   = re.compile('(\d+)?\s+?%s\s*((\d+)(/\d+)?)?\s*$' % self._fmon_str,
-                                    re.IGNORECASE)
-        self._ptext    = re.compile('%s\s+(\d+)?\s*,?\s*((\d+)(/\d+)?)?\s*$' % self._pmon_str,
-                                    re.IGNORECASE)
-        self._ptext2   = re.compile('(\d+)?\s+?%s\s*((\d+)(/\d+)?)?\s*$' % self._pmon_str,
-                                    re.IGNORECASE)
-        self._itext    = re.compile('%s\s+(\d+)?\s*,?\s*((\d+)(/\d+)?)?\s*$' % self._imon_str,
-                                    re.IGNORECASE)
-        self._itext2   = re.compile('(\d+)?\s+?%s\s*((\d+)(/\d+)?)?\s*$' % self._imon_str,
-                                    re.IGNORECASE)
-        self._stext     = re.compile('%s\.?\s+(\d+)?\s*,?\s*((\d+)(/\d+)?)?\s*$' % self._smon_str,
-                                    re.IGNORECASE)
-        self._stext2    = re.compile('(\d+)?\s+?%s\.?\s*((\d+)(/\d+)?)?\s*$' % self._smon_str,
-                                    re.IGNORECASE)
-        self._numeric  = re.compile("((\d+)[/\.]\s*)?((\d+)[/\.]\s*)?(\d+)\s*$")
-        self._iso      = re.compile("(\d+)(/(\d+))?-(\d+)-(\d+)\s*$")
-        self._isotimestamp = re.compile("^\s*?(\d{4})([01]\d)([0123]\d)(?:(?:[012]\d[0-5]\d[0-5]\d)|(?:\s+[012]\d:[0-5]\d(?::[0-5]\d)?))?\s*?$")
-        self._rfc      = re.compile("(%s,)?\s+(\d|\d\d)\s+%s\s+(\d+)\s+\d\d:\d\d(:\d\d)?\s+(\+|-)\d\d\d\d"
-                                    % (self._rfc_day_str, self._rfc_mon_str))
-        self._today = re.compile("^\s*%s\s*$" % self._today_str, re.IGNORECASE)
+        self._text2 = re.compile(r'(\d+)?\s+?%s\.?\s*((\d+)(/\d+)?)?\s*$'
+                                 % self._mon_str, re.IGNORECASE)
+        self._jtext = re.compile(r'%s\s+(\d+)?\s*,?\s*((\d+)(/\d+)?)?\s*$'
+                                 % self._jmon_str, re.IGNORECASE)
+        self._jtext2 = re.compile(r'(\d+)?\s+?%s\s*((\d+)(/\d+)?)?\s*$'
+                                  % self._jmon_str, re.IGNORECASE)
+        self._ftext = re.compile(r'%s\s+(\d+)?\s*,?\s*((\d+)(/\d+)?)?\s*$'
+                                 % self._fmon_str, re.IGNORECASE)
+        self._ftext2 = re.compile(r'(\d+)?\s+?%s\s*((\d+)(/\d+)?)?\s*$'
+                                  % self._fmon_str, re.IGNORECASE)
+        self._ptext = re.compile(r'%s\s+(\d+)?\s*,?\s*((\d+)(/\d+)?)?\s*$'
+                                 % self._pmon_str, re.IGNORECASE)
+        self._ptext2 = re.compile(r'(\d+)?\s+?%s\s*((\d+)(/\d+)?)?\s*$'
+                                  % self._pmon_str, re.IGNORECASE)
+        self._itext = re.compile(r'%s\s+(\d+)?\s*,?\s*((\d+)(/\d+)?)?\s*$'
+                                 % self._imon_str, re.IGNORECASE)
+        self._itext2 = re.compile(r'(\d+)?\s+?%s\s*((\d+)(/\d+)?)?\s*$'
+                                  % self._imon_str, re.IGNORECASE)
+        self._stext = re.compile(r'%s\.?\s+(\d+)?\s*,?\s*((\d+)(/\d+)?)?\s*$'
+                                 % self._smon_str, re.IGNORECASE)
+        self._stext2 = re.compile(r'(\d+)?\s+?%s\.?\s*((\d+)(/\d+)?)?\s*$'
+                                  % self._smon_str, re.IGNORECASE)
+        self._numeric = re.compile(
+            r"((\d+)[/\.]\s*)?((\d+)[/\.]\s*)?(\d+)\s*$")
+        self._iso = re.compile(r"(\d+)(/(\d+))?-(\d+)-(\d+)\s*$")
+        self._isotimestamp = re.compile(
+            r"^\s*?(\d{4})([01]\d)([0123]\d)(?:(?:[012]\d[0-5]\d[0-5]\d)|"
+            r"(?:\s+[012]\d:[0-5]\d(?::[0-5]\d)?))?\s*?$")
+        self._rfc = re.compile(
+            r"(%s,)?\s+(\d|\d\d)\s+%s\s+(\d+)\s+\d\d:\d\d(:\d\d)?\s+"
+            r"(\+|-)\d\d\d\d" % (self._rfc_day_str, self._rfc_mon_str))
+        self._today = re.compile(r"^\s*%s\s*$" % self._today_str,
+                                 re.IGNORECASE)
 
     def _get_int(self, val):
         """
@@ -905,7 +911,7 @@ class DateParser:
         text = text.strip() # otherwise spaces can make it a bad date
         date.set_text_value(text)
         qual = Date.QUAL_NONE
-        cal  = Date.CAL_GREGORIAN
+        cal = Date.CAL_GREGORIAN
         newyear = Date.NEWYEAR_JAN1
 
         (text, cal, newyear) = self.match_calendar_newyear(text, cal, newyear)
