@@ -32,7 +32,7 @@ This module provides the model that is used for all hierarchical treeviews.
 # Standard python modules
 #
 #-------------------------------------------------------------------------
-import time
+from time import perf_counter
 import logging
 
 _LOG = logging.getLogger(".gui.treebasemodel")
@@ -47,22 +47,24 @@ from gi.repository import Gtk
 
 #-------------------------------------------------------------------------
 #
-# GRAMPS modules
+# Gramps modules
 #
 #-------------------------------------------------------------------------
 from gramps.gen.const import GRAMPS_LOCALE as glocale
 _ = glocale.translation.gettext
 import gramps.gui.widgets.progressdialog as progressdlg
+from ...user import User
 from bisect import bisect_right
 from gramps.gen.filters import SearchFilter, ExactSearchFilter
 from .basemodel import BaseModel
+from gramps.gen.proxy.cache import CacheProxyDb
 
 #-------------------------------------------------------------------------
 #
 # Node
 #
 #-------------------------------------------------------------------------
-class Node(object):
+class Node:
     """
     This class defines an individual node of a tree in the model.  The node
     stores the following data:
@@ -104,7 +106,7 @@ class Node(object):
         """
         Assign the handle of a Gramps object to this node.
         """
-        if not self.handle or handle == None:
+        if not self.handle or handle is None:
             self.handle = handle
             self.secondary = secondary
         else:
@@ -174,7 +176,7 @@ class Node(object):
 # NodeMap
 #
 #-------------------------------------------------------------------------
-class NodeMap(object):
+class NodeMap:
     """
     Map of id of Node classes to real object
     """
@@ -274,15 +276,13 @@ class TreeBaseModel(GObject.GObject, Gtk.TreeModel, BaseModel):
                       secondary object type.
     """
 
-    def __init__(self, db,
-                    search=None, skip=set(),
-                    scol=0, order=Gtk.SortType.ASCENDING, sort_map=None,
-                    nrgroups = 1,
-                    group_can_have_handle = False,
-                    has_secondary=False):
-        cput = time.clock()
+    def __init__(self, db, uistate, search=None, skip=set(), scol=0,
+                 order=Gtk.SortType.ASCENDING, sort_map=None, nrgroups = 1,
+                 group_can_have_handle = False, has_secondary=False):
+        cput = perf_counter()
         GObject.GObject.__init__(self)
         BaseModel.__init__(self)
+
         #We create a stamp to recognize invalid iterators. From the docs:
         #Set the stamp to be equal to your model's stamp, to mark the
         #iterator as valid. When your model's structure changes, you should
@@ -294,12 +294,14 @@ class TreeBaseModel(GObject.GObject, Gtk.TreeModel, BaseModel):
         self.prev_handle = None
         self.prev_data = None
 
+        self.uistate = uistate
         self.__reverse = (order == Gtk.SortType.DESCENDING)
         self.scol = scol
         self.nrgroups = nrgroups
         self.group_can_have_handle = group_can_have_handle
         self.has_secondary = has_secondary
         self.db = db
+        self.dont_change_active = False
 
         self._set_base_data()
 
@@ -340,7 +342,7 @@ class TreeBaseModel(GObject.GObject, Gtk.TreeModel, BaseModel):
             self.rebuild_data(self.current_filter, skip=skip)
 
         _LOG.debug(self.__class__.__name__ + ' __init__ ' +
-                    str(time.clock() - cput) + ' sec')
+                    str(perf_counter() - cput) + ' sec')
 
     def destroy(self):
         """
@@ -483,11 +485,11 @@ class TreeBaseModel(GObject.GObject, Gtk.TreeModel, BaseModel):
         the filter functions. When called internally (from __init__) both
         data_filter and data_filter2 will have been set from set_search
         """
-        cput = time.clock()
+        cput = perf_counter()
         self.clear_cache()
         self._in_build = True
 
-        if not self.db.is_open():
+        if not ((self.db is not None) and self.db.is_open()):
             return
 
         self.clear()
@@ -503,7 +505,7 @@ class TreeBaseModel(GObject.GObject, Gtk.TreeModel, BaseModel):
             self.current_filter2 = data_filter2
 
         _LOG.debug(self.__class__.__name__ + ' rebuild_data ' +
-                    str(time.clock() - cput) + ' sec')
+                    str(perf_counter() - cput) + ' sec')
 
     def _rebuild_search(self, dfilter, dfilter2, skip):
         """
@@ -528,28 +530,22 @@ class TreeBaseModel(GObject.GObject, Gtk.TreeModel, BaseModel):
         Rebuild the data map for a single Gramps object type, where a search
         condition is applied.
         """
-        pmon = progressdlg.ProgressMonitor(progressdlg.GtkProgressDialog,
-                                            popup_time=2)
-        status = progressdlg.LongOpStatus(msg=_("Building View"),
-                            total_steps=items, interval=items//20,
-                            can_cancel=True)
+        pmon = progressdlg.ProgressMonitor(
+            progressdlg.StatusProgress, (self.uistate,), popup_time=2,
+            title=_("Loading items..."))
+        status = progressdlg.LongOpStatus(total_steps=items,
+                                          interval=items // 20)
         pmon.add_op(status)
         with gen_cursor() as cursor:
             for handle, data in cursor:
-                # for python3 this returns a byte object, so conversion needed
-                if not isinstance(handle, str):
-                    handle = handle.decode('utf-8')
                 status.heartbeat()
-                if status.should_cancel():
-                    break
                 self.__total += 1
                 if not (handle in skip or (dfilter and not
                                         dfilter.match(handle, self.db))):
                     _LOG.debug("    add %s %s" % (handle, data))
                     self.__displayed += 1
                     add_func(handle, data)
-        if not status.was_cancelled():
-            status.end()
+        status.end()
 
     def _rebuild_filter(self, dfilter, dfilter2, skip):
         """
@@ -579,28 +575,31 @@ class TreeBaseModel(GObject.GObject, Gtk.TreeModel, BaseModel):
         Rebuild the data map for a single Gramps object type, where a filter
         is applied.
         """
-        pmon = progressdlg.ProgressMonitor(progressdlg.GtkProgressDialog,
-                                            popup_time=2)
-        status = progressdlg.LongOpStatus(msg=_("Building View"),
-                              total_steps=3, interval=1)
-        pmon.add_op(status)
-        status_ppl = progressdlg.LongOpStatus(msg=_("Loading items..."),
-                        total_steps=items, interval=items//10)
+        pmon = progressdlg.ProgressMonitor(
+            progressdlg.StatusProgress, (self.uistate,), popup_time=2,
+            title=_("Loading items..."))
+        status_ppl = progressdlg.LongOpStatus(total_steps=items,
+                                              interval=items // 20)
         pmon.add_op(status_ppl)
 
         self.__total += items
-
-        with gen_cursor() as cursor:
-            for handle, data in cursor:
-                if not isinstance(handle, str):
-                    handle = handle.decode('utf-8')
+        assert not skip
+        if dfilter:
+            cdb = CacheProxyDb(self.db)
+            for handle in dfilter.apply(cdb, tree=True,
+                                        user=User(parent=self.uistate.window)):
                 status_ppl.heartbeat()
-                if not handle in skip:
-                    if not dfilter or dfilter.match(handle, self.db):
-                        add_func(handle, data)
-                        self.__displayed += 1
+                data = data_map(handle)
+                add_func(handle, data)
+                self.__displayed += 1
+        else:
+            with gen_cursor() as cursor:
+                for handle, data in cursor:
+                    status_ppl.heartbeat()
+                    add_func(handle, data)
+                    self.__displayed += 1
+
         status_ppl.end()
-        status.end()
 
     def add_node(self, parent, child, sortkey, handle, add_parent=True,
                  secondary=False):
@@ -746,7 +745,7 @@ class TreeBaseModel(GObject.GObject, Gtk.TreeModel, BaseModel):
         self.clear_path_cache()
         if self._get_node(handle) is not None:
             return # row already exists
-        cput = time.clock()
+        cput = perf_counter()
         data = self.map(handle)
         if data:
             if not self.search or \
@@ -758,7 +757,7 @@ class TreeBaseModel(GObject.GObject, Gtk.TreeModel, BaseModel):
                 self.add_row2(handle, self.map2(handle))
 
         _LOG.debug(self.__class__.__name__ + ' add_row_by_handle ' +
-                    str(time.clock() - cput) + ' sec')
+                    str(perf_counter() - cput) + ' sec')
         _LOG.debug("displayed %d / total: %d" %
                    (self.__displayed, self.__total))
 
@@ -767,7 +766,7 @@ class TreeBaseModel(GObject.GObject, Gtk.TreeModel, BaseModel):
         Delete a row from the model.
         """
         assert isinstance(handle, str)
-        cput = time.clock()
+        cput = perf_counter()
         self.clear_cache(handle)
         node = self._get_node(handle)
         if node is None:
@@ -789,25 +788,33 @@ class TreeBaseModel(GObject.GObject, Gtk.TreeModel, BaseModel):
             parent = next_parent
 
         _LOG.debug(self.__class__.__name__ + ' delete_row_by_handle ' +
-                    str(time.clock() - cput) + ' sec')
+                    str(perf_counter() - cput) + ' sec')
         _LOG.debug("displayed %d / total: %d" %
                    (self.__displayed, self.__total))
 
     def update_row_by_handle(self, handle):
         """
         Update a row in the model.
+
+        We have to do delete/add because sometimes row position changes when
+        object name changes.
+        A delete action causes the listview module to set a prior row to
+        active.  In some cases (merge) the prior row may have been already
+        removed from the db.  To avoid invalid handle exceptions in gramplets
+        at the change active, we tell listview not to change active.
+        The add_row below changes to current active again so we end up in right
+        place.
         """
         assert isinstance(handle, str)
         self.clear_cache(handle)
         if self._get_node(handle) is None:
-            return # row not currently displayed
+            return  # row not currently displayed
 
+        self.dont_change_active = True
         self.delete_row_by_handle(handle)
+        self.dont_change_active = False
         self.add_row_by_handle(handle)
 
-        # If the node hasn't moved, all we need is to call row_changed.
-        #self.row_changed(path, node)
-
     def _new_iter(self, nodeid):
         """
         Return a new iter containing the nodeid in the nodemap
@@ -857,10 +864,7 @@ class TreeBaseModel(GObject.GObject, Gtk.TreeModel, BaseModel):
         not correspond to a gramps object.
         """
         node = self.get_node_from_iter(iter)
-        handle = node.handle
-        if handle and not isinstance(handle, str):
-            handle = handle.decode('utf-8')
-        return handle
+        return node.handle
 
     # The following implement the public interface of Gtk.TreeModel
 
@@ -897,7 +901,7 @@ class TreeBaseModel(GObject.GObject, Gtk.TreeModel, BaseModel):
             # Header rows dont get the foreground color set
             if col == self.color_column():
                 #color must not be utf-8
-                return "#000000000000"
+                return ""
 
             # Return the node name for the first column
             if col == 0:
@@ -909,14 +913,10 @@ class TreeBaseModel(GObject.GObject, Gtk.TreeModel, BaseModel):
             # return values for 'data' row, calling a function
             # according to column_defs table
             val = self._get_value(node.handle, col, node.secondary)
-        #GTK 3 should convert unicode objects automatically, but this
-        # gives wrong column values, so convert for python 2.7
+
         if val is None:
             return ''
-        elif not isinstance(val, str):
-            return val.encode('utf-8')
-        else:
-            return val
+        return val
 
     def _get_value(self, handle, col, secondary=False, store_cache=True):
         """
