@@ -61,8 +61,8 @@ from gramps.gen.lib import Date, Person
 from gramps.gen.updatecallback import UpdateCallback
 from gramps.gen.db.exceptions import DbWriteFailure
 from gramps.version import VERSION
-from gramps.gen.constfunc import win, conv_to_unicode
-from gramps.gui.plug.export import WriterOptionBox
+from gramps.gen.constfunc import win
+from gramps.gui.plug.export import WriterOptionBox, WriterOptionBoxWithCompression
 import gramps.plugins.lib.libgrampsxml as libgrampsxml
 
 #-------------------------------------------------------------------------
@@ -81,7 +81,7 @@ except:
 strip_dict = dict.fromkeys(list(range(9))+list(range(11,13))+list(range(14, 32)))
 
 def escxml(d):
-    return escape(d, 
+    return escape(d,
                   {'"' : '&quot;',
                    '<' : '&lt;',
                    '>' : '&gt;',
@@ -126,7 +126,10 @@ class GrampsXmlWriter(UpdateCallback):
         """
         if filename == '-':
             import sys
-            g = sys.stdout.buffer
+            try:
+                g = sys.stdout.buffer
+            except:
+                g = sys.stdout
             self.compress = False
         else:
             base = os.path.dirname(filename)
@@ -145,7 +148,7 @@ class GrampsXmlWriter(UpdateCallback):
                                        'Please select another directory '
                                        'or create it.') % base )
                 return 0
-                
+
             if os.path.exists(filename):
                 if not os.access(filename, os.W_OK):
                     raise DbWriteFailure(
@@ -155,18 +158,18 @@ class GrampsXmlWriter(UpdateCallback):
                             "Please make sure you have write access to the "
                             "file and try again."))
                     return 0
-        
+
             self.fileroot = os.path.dirname(filename)
             try:
                 if self.compress and _gzip_ok:
                     try:
                         g = gzip.open(filename,"wb")
                     except:
-                        g = open(filename,"w")
+                        g = open(filename,"wb")
                 else:
-                    g = open(filename,"w")
+                    g = open(filename,"wb")
             except IOError as msg:
-                LOG.warn(str(msg))
+                LOG.warning(str(msg))
                 raise DbWriteFailure(_('Failure writing %s') % filename,
                                         str(msg))
                 return 0
@@ -196,7 +199,7 @@ class GrampsXmlWriter(UpdateCallback):
         self.write_xml_data()
         g.close()
         return 1
-            
+
     def write_xml_data(self):
 
         date = time.localtime(time.time())
@@ -209,17 +212,17 @@ class GrampsXmlWriter(UpdateCallback):
         source_len = self.db.get_number_of_sources()
         place_len = self.db.get_number_of_places()
         repo_len = self.db.get_number_of_repositories()
-        obj_len = self.db.get_number_of_media_objects()
-        note_len = self.db.get_number_of_notes()        
-        tag_len = self.db.get_number_of_tags()        
-        
+        obj_len = self.db.get_number_of_media()
+        note_len = self.db.get_number_of_notes()
+        tag_len = self.db.get_number_of_tags()
+
         total_steps = (person_len + family_len + event_len + citation_len +
                        source_len + place_len + repo_len + obj_len + note_len +
                        tag_len
                       )
 
         self.set_total(total_steps)
-        
+
         self.g.write('<?xml version="1.0" encoding="UTF-8"?>\n')
         self.g.write('<!DOCTYPE database '
                      'PUBLIC "-//Gramps//DTD Gramps XML %s//EN"\n'
@@ -255,7 +258,8 @@ class GrampsXmlWriter(UpdateCallback):
             self.g.write("  <tags>\n")
             for key in sorted(self.db.get_tag_handles()):
                 tag = self.db.get_tag_from_handle(key)
-                self.write_tag(tag, 2)
+                if tag:
+                    self.write_tag(tag, 2)
                 self.update()
             self.g.write("  </tags>\n")
 
@@ -264,7 +268,8 @@ class GrampsXmlWriter(UpdateCallback):
             self.g.write("  <events>\n")
             for handle in sorted(self.db.get_event_handles()):
                 event = self.db.get_event_from_handle(handle)
-                self.write_event(event,2)
+                if event:
+                    self.write_event(event,2)
                 self.update()
             self.g.write("  </events>\n")
 
@@ -277,7 +282,8 @@ class GrampsXmlWriter(UpdateCallback):
 
             for handle in sorted(self.db.get_person_handles()):
                 person = self.db.get_person_from_handle(handle)
-                self.write_person(person, 2)
+                if person:
+                    self.write_person(person, 2)
                 self.update()
             self.g.write("  </people>\n")
 
@@ -285,7 +291,8 @@ class GrampsXmlWriter(UpdateCallback):
             self.g.write("  <families>\n")
             for handle in sorted(self.db.iter_family_handles()):
                 family = self.db.get_family_from_handle(handle)
-                self.write_family(family,2)
+                if family:
+                    self.write_family(family,2)
                 self.update()
             self.g.write("  </families>\n")
 
@@ -293,7 +300,8 @@ class GrampsXmlWriter(UpdateCallback):
             self.g.write("  <citations>\n")
             for handle in sorted(self.db.get_citation_handles()):
                 citation = self.db.get_citation_from_handle(handle)
-                self.write_citation(citation,2)
+                if citation:
+                    self.write_citation(citation,2)
                 self.update()
             self.g.write("  </citations>\n")
 
@@ -301,7 +309,8 @@ class GrampsXmlWriter(UpdateCallback):
             self.g.write("  <sources>\n")
             for handle in sorted(self.db.get_source_handles()):
                 source = self.db.get_source_from_handle(handle)
-                self.write_source(source,2)
+                if source:
+                    self.write_source(source,2)
                 self.update()
             self.g.write("  </sources>\n")
 
@@ -310,15 +319,17 @@ class GrampsXmlWriter(UpdateCallback):
             for key in sorted(self.db.get_place_handles()):
                 # try:
                 place = self.db.get_place_from_handle(key)
-                self.write_place_obj(place,2)
+                if place:
+                    self.write_place_obj(place,2)
                 self.update()
             self.g.write("  </places>\n")
 
         if obj_len > 0:
             self.g.write("  <objects>\n")
-            for handle in sorted(self.db.get_media_object_handles()):
-                obj = self.db.get_object_from_handle(handle)
-                self.write_object(obj,2)
+            for handle in sorted(self.db.get_media_handles()):
+                obj = self.db.get_media_from_handle(handle)
+                if obj:
+                    self.write_object(obj,2)
                 self.update()
             self.g.write("  </objects>\n")
 
@@ -326,7 +337,8 @@ class GrampsXmlWriter(UpdateCallback):
             self.g.write("  <repositories>\n")
             for key in sorted(self.db.get_repository_handles()):
                 repo = self.db.get_repository_from_handle(key)
-                self.write_repository(repo,2)
+                if repo:
+                    self.write_repository(repo,2)
                 self.update()
             self.g.write("  </repositories>\n")
 
@@ -334,7 +346,8 @@ class GrampsXmlWriter(UpdateCallback):
             self.g.write("  <notes>\n")
             for key in sorted(self.db.get_note_handles()):
                 note = self.db.get_note_from_handle(key)
-                self.write_note(note, 2)
+                if note:
+                    self.write_note(note, 2)
                 self.update()
             self.g.write("  </notes>\n")
 
@@ -343,7 +356,7 @@ class GrampsXmlWriter(UpdateCallback):
         self.write_namemaps()
 
         self.g.write("</database>\n")
-        
+
 #        self.status.end()
 #        self.status = None
 
@@ -357,7 +370,7 @@ class GrampsXmlWriter(UpdateCallback):
     def write_namemaps(self):
         group_map = self.db.get_name_group_keys()
         name_len = len(group_map)
-        
+
         if name_len > 0:
             self.g.write("  <namemaps>\n")
             for key in group_map:
@@ -381,7 +394,7 @@ class GrampsXmlWriter(UpdateCallback):
                   bm_source_len + bm_place_len + bm_repo_len +
                   bm_citation_len + bm_obj_len + bm_note_len
                   )
-        
+
         if bm_len > 0:
             self.g.write("  <bookmarks>\n")
 
@@ -421,7 +434,7 @@ class GrampsXmlWriter(UpdateCallback):
             for number, name,fmt_str,active in self.db.name_formats:
                 self.g.write('%s<format number="%d" name="%s" '
                              'fmt_str="%s" active="%d"/>\n'
-                             % ('    ', number, 
+                             % ('    ', number,
                                 escxml(name), escxml(fmt_str), int(active)) )
             self.g.write("  </name-formats>\n")
 
@@ -437,12 +450,9 @@ class GrampsXmlWriter(UpdateCallback):
         self.g.write(' color="%s"' % tag.get_color())
         self.g.write(' priority="%d"' % tag.get_priority())
         self.g.write('/>\n')
-        
-    def fix(self,line):
-        try:
-            l = str(line)
-        except:
-            l = conv_to_unicode(str(line), errors='replace')
+
+    def fix(self, line):
+        l = str(line)
         l = l.strip().translate(strip_dict)
         return escxml(l)
 
@@ -466,7 +476,7 @@ class GrampsXmlWriter(UpdateCallback):
         if format != note.FLOWED:
             self.g.write(' format="%d"' % format)
         self.g.write('>\n')
-        
+
         self.write_text('text', text, index + 1)
 
         if styles:
@@ -476,32 +486,32 @@ class GrampsXmlWriter(UpdateCallback):
             self.write_ref("tagref", tag_handle, index+1)
 
         self.g.write('  ' * index + '</note>\n')
-        
+
     def write_styles(self, styles, index=3):
         for style in styles:
             name = style.name.xml_str()
             value = style.value
-            
+
             self.g.write('  ' * index + '<style name="%s"' % name)
             if value:
                 self.g.write(' value="%s"' % escxml(str(value)))
             self.g.write('>\n')
-            
+
             for (start, end) in style.ranges:
                 self.g.write(('  ' * (index + 1)) +
                              '<range start="%d" end="%d"/>\n' % (start, end))
-            
+
             self.g.write('  ' * index + '</style>\n')
-    
+
     def write_text(self, val, text, indent=0):
         if not text:
             return
-        
+
         if indent:
             self.g.write('  ' * indent)
-        
+
         self.g.write('<%s>' % val)
-        self.g.write(self.fix(text.rstrip()))
+        self.g.write(escxml(str(text).translate(strip_dict)))
         self.g.write("</%s>\n" % val)
 
     def write_person(self,person,index=1):
@@ -697,7 +707,7 @@ class GrampsXmlWriter(UpdateCallback):
                 self.write_ref("citationref", citation_handle, index+1)
             self.write_note_list(nreflist,index+1)
             self.g.write('%s</childref>\n' % sp)
-        
+
     def dump_event_ref(self,eventref,index=1):
         if not eventref or not eventref.ref:
             return
@@ -760,7 +770,7 @@ class GrampsXmlWriter(UpdateCallback):
         self.write_line("description",event.get_description(),index+1)
         self.write_attribute_list(event.get_attribute_list(),index+1)
         self.write_note_list(event.get_note_list(),index+1)
-            
+
         for citation_handle in event.get_citation_list():
             self.write_ref("citationref", citation_handle, index+1)
         self.write_media_list(event.get_media_list(),index+1)
@@ -789,7 +799,7 @@ class GrampsXmlWriter(UpdateCallback):
         if ord.get_status() != 0:
             self.g.write('%s<status val="%s"/>\n' % (sp2, ord.status2xml()))
         if ord.get_family_handle():
-            self.g.write('%s<sealed_to hlink="%s"/>\n' % 
+            self.g.write('%s<sealed_to hlink="%s"/>\n' %
                          (sp2,"_"+ord.get_family_handle()))
         self.write_note_list(ord.get_note_list(),index+1)
         for citation_handle in ord.get_citation_list():
@@ -831,9 +841,9 @@ class GrampsXmlWriter(UpdateCallback):
             change_text = ' change="%d"' %  obj.get_change_time()
         except:
             change_text = ' change="%d"' %  0
-            
+
         handle_text = ' handle="_%s"' % obj.get_handle()
-        
+
         obj_text = '%s<%s' % (sp, tagname)
         self.g.write(obj_text + handle_text + change_text)
         if close:
@@ -865,7 +875,7 @@ class GrampsXmlWriter(UpdateCallback):
             self.g.write(' connector="%s"' % escxml(con))
         if der:
             self.g.write(' derivation="%s"' % escxml(der))
-            
+
         self.g.write('>%s</surname>\n' % self.fix(nam))
 
     def write_line(self,tagname,value,indent=1):
@@ -874,7 +884,7 @@ class GrampsXmlWriter(UpdateCallback):
                          ('  '*indent,tagname,self.fix(value),tagname))
 
     def write_line_nofix(self,tagname,value,indent=1):
-        """Writes a line, but does not escape characters. 
+        """Writes a line, but does not escape characters.
             Use this instead of write_line if the value is already fixed,
             this avoids &amp; becoming &amp;amp;
         """
@@ -892,7 +902,7 @@ class GrampsXmlWriter(UpdateCallback):
             y = "????"
         else:
             y = "%04d" % date[2]
-            
+
         if date[1] == 0:
             if date[0] == 0:
                 m = ""
@@ -927,7 +937,7 @@ class GrampsXmlWriter(UpdateCallback):
             qual_str = ' quality="calculated"'
         else:
             qual_str = ""
-            
+
         dualdated = date.get_slash()
         if dualdated:
             dualdated_str = ' dualdated="1"'
@@ -941,12 +951,12 @@ class GrampsXmlWriter(UpdateCallback):
             newyear_str = ''
 
         mode = date.get_modifier()
-        
+
         if date.is_compound():
             if mode == Date.MOD_RANGE:
                 tagname = 'daterange'
             else:
-                tagname = 'datespan'         
+                tagname = 'datespan'
 
             d1 = self.get_iso_date(date.get_start_date())
             d2 = self.get_iso_date(date.get_stop_date())
@@ -958,7 +968,7 @@ class GrampsXmlWriter(UpdateCallback):
             date_str = self.get_iso_date(date.get_start_date())
             if date_str == "":
                 return
-            
+
             if mode == Date.MOD_BEFORE:
                 mode_str = ' type="before"'
             elif mode == Date.MOD_AFTER:
@@ -1013,7 +1023,7 @@ class GrampsXmlWriter(UpdateCallback):
         self.write_note_list(name.get_note_list(),index+1)
         for citation_handle in name.get_citation_list():
             self.write_ref("citationref", citation_handle, index+1)
-    
+
         self.g.write('%s</name>\n' % sp)
 
     def append_value(self, orig,val):
@@ -1063,7 +1073,7 @@ class GrampsXmlWriter(UpdateCallback):
         country = self.fix(loc.get_country())
         zip_code = self.fix(loc.get_postal_code())
         phone = self.fix(loc.get_phone())
-        
+
         self.g.write('      <location')
         if street:
             self.g.write(' street="%s"' % street)
@@ -1172,7 +1182,7 @@ class GrampsXmlWriter(UpdateCallback):
                 callno_text = ''
             else:
                 callno_text = ' callno="%s"' % escxml(reporef.call_number)
-                
+
             mtype = reporef.media_type.xml_str()
             if mtype:
                 type_text = ' medium="%s"' % escxml(mtype)
@@ -1188,8 +1198,8 @@ class GrampsXmlWriter(UpdateCallback):
                                extra_text=priv_text+callno_text+type_text)
                 self.write_note_list(note_list, index+1)
                 sp = "  "*index
-                self.g.write('%s</reporef>\n' % sp)            
-            
+                self.g.write('%s</reporef>\n' % sp)
+
     def write_url_list(self, list, index=1):
         sp = "  "*index
         for url in list:
@@ -1270,10 +1280,10 @@ class GrampsXmlWriter(UpdateCallback):
             path = path[1:]
         if win():
             # Always export path with \ replaced with /. Otherwise import
-            # from Windows to Linux of gpkg's path to images does not work. 
+            # from Windows to Linux of gpkg's path to images does not work.
             path = path.replace('\\','/')
         self.g.write('%s<file src="%s" mime="%s"%s%s/>\n'
-                     % ("  "*(index+1), self.fix(path), self.fix(mime_type), 
+                     % ("  "*(index+1), self.fix(path), self.fix(mime_type),
                         checksum_text, desc_text))
         self.write_attribute_list(obj.get_attribute_list())
         self.write_note_list(obj.get_note_list(), index+1)
@@ -1334,6 +1344,7 @@ def export_data(database, filename, user, option_box=None):
     if option_box:
         option_box.parse_options()
         database = option_box.get_filtered_database(database)
+        compress = compress and option_box.get_use_compression()
 
     g = XmlWriter(database, user, 0, compress)
     return g.write(filename)
