@@ -48,7 +48,7 @@ _LOG = logging.getLogger(".gui.personview")
 from gramps.gen.lib import Person, Surname
 from gramps.gen.db import DbTxn
 from gramps.gui.views.listview import ListView, TEXT, MARKUP, ICON
-from gramps.gui.actiongroup import ActionGroup
+from gramps.gui.uimanager import ActionGroup
 from gramps.gen.utils.string import data_recover_msg
 from gramps.gen.display.name import displayer as name_displayer
 from gramps.gui.dialog import ErrorDialog, MultiSelectDialog, QuestionDialog
@@ -120,11 +120,11 @@ class BasePersonView(ListView):
                            COL_TAGS, COL_CHAN]),
         ('columns.size', [250, 75, 75, 100, 175, 100, 175, 100, 30, 30, 30, 30,
                           30, 100, 100])
-        )  
-    ADD_MSG     = _("Add a new person")
-    EDIT_MSG    = _("Edit the selected person")
-    DEL_MSG     = _("Remove the selected person")
-    MERGE_MSG   = _("Merge the selected persons")
+        )
+    ADD_MSG = _("Add a new person")
+    EDIT_MSG = _("Edit the selected person")
+    DEL_MSG = _("Delete the selected person")
+    MERGE_MSG = _("Merge the selected persons")
     FILTER_TYPE = "Person"
     QR_CATEGORY = CATEGORY_QR_PERSON
 
@@ -139,23 +139,23 @@ class BasePersonView(ListView):
             'person-rebuild' : self.object_build,
             'person-groupname-rebuild' : self.object_build,
             'no-database': self.no_database,
+            'family-update'  : self.related_update,
+            'family-add'     : self.related_update,
+            'event-update'   : self.related_update,
+            'place-update'   : self.related_update,
             }
- 
+
         ListView.__init__(
             self, title, pdata, dbstate, uistate,
             model, signal_map,
             PersonBookmarks, nav_group,
             multiple=True,
             filter_class=PersonSidebarFilter)
-            
-        self.func_list.update({
-            '<PRIMARY>J' : self.jump,
-            '<PRIMARY>BackSpace' : self.key_delete,
-            })
 
         uistate.connect('nameformat-changed', self.build_tree)
+        uistate.connect('placeformat-changed', self.build_tree)
 
-        self.additional_uis.append(self.additional_ui())
+        self.additional_uis.append(self.additional_ui)
 
     def navigation_type(self):
         """
@@ -168,7 +168,7 @@ class BasePersonView(ListView):
         Specify the drag type for a single selection
         """
         return DdTargets.PERSON_LINK
-        
+
     def exact_search(self):
         """
         Returns a tuple indicating columns requiring an exact search
@@ -182,74 +182,230 @@ class BasePersonView(ListView):
         """
         return 'gramps-person'
 
-    def additional_ui(self):
-        """
-        Defines the UI string for UIManager
-        """
-        return '''<ui>
-          <menubar name="MenuBar">
-            <menu action="FileMenu">
-              <placeholder name="LocalExport">
-                <menuitem action="ExportTab"/>
-              </placeholder>
-            </menu>
-            <menu action="BookMenu">
-              <placeholder name="AddEditBook">
-                <menuitem action="AddBook"/>
-                <menuitem action="EditBook"/>
-              </placeholder>
-            </menu>
-            <menu action="GoMenu">
-              <placeholder name="CommonGo">
-                <menuitem action="Back"/>
-                <menuitem action="Forward"/>
-                <separator/>
-                <menuitem action="HomePerson"/>
-                <separator/>
-              </placeholder>
-            </menu>
-            <menu action="EditMenu">
-              <placeholder name="CommonEdit">
-                <menuitem action="Add"/>
-                <menuitem action="Edit"/>
-                <menuitem action="Remove"/>
-                <menuitem action="Merge"/>
-              </placeholder>
-              <menuitem action="SetActive"/>
-              <menuitem action="FilterEdit"/>
-            </menu>
-          </menubar>
-          <toolbar name="ToolBar">
-            <placeholder name="CommonNavigation">
-              <toolitem action="Back"/>
-              <toolitem action="Forward"/>  
-              <toolitem action="HomePerson"/>
-            </placeholder>
-            <placeholder name="CommonEdit">
-              <toolitem action="Add"/>
-              <toolitem action="Edit"/>
-              <toolitem action="Remove"/>
-              <toolitem action="Merge"/>
-            </placeholder>
-          </toolbar>
-          <popup name="Popup">
-            <menuitem action="Back"/>
-            <menuitem action="Forward"/>
-            <menuitem action="HomePerson"/>
-            <separator/>
-            <menuitem action="Add"/>
-            <menuitem action="Edit"/>
-            <menuitem action="Remove"/>
-            <menuitem action="Merge"/>
-            <separator/>
-            <menu name="QuickReport" action="QuickReport"/>
-            <menu name="WebConnect" action="WebConnect"/>
-          </popup>
-        </ui>'''
+    additional_ui = [  # Defines the UI string for UIManager
+        '''
+      <placeholder id="LocalExport">
+        <item>
+          <attribute name="action">win.ExportTab</attribute>
+          <attribute name="label" translatable="yes">Export View...</attribute>
+        </item>
+      </placeholder>
+''',
+        '''
+      <section id="AddEditBook">
+        <item>
+          <attribute name="action">win.AddBook</attribute>
+          <attribute name="label" translatable="yes">_Add Bookmark</attribute>
+        </item>
+        <item>
+          <attribute name="action">win.EditBook</attribute>
+          <attribute name="label" translatable="no">%s...</attribute>
+        </item>
+      </section>
+''' % _('Organize Bookmarks'),
+        '''
+      <placeholder id="CommonGo">
+      <section>
+        <item>
+          <attribute name="action">win.Back</attribute>
+          <attribute name="label" translatable="yes">_Back</attribute>
+        </item>
+        <item>
+          <attribute name="action">win.Forward</attribute>
+          <attribute name="label" translatable="yes">_Forward</attribute>
+        </item>
+      </section>
+      <section>
+        <item>
+          <attribute name="action">win.HomePerson</attribute>
+          <attribute name="label" translatable="yes">_Home</attribute>
+        </item>
+      </section>
+      </placeholder>
+''',
+        '''
+      <section id='CommonEdit' groups='RW'>
+        <item>
+          <attribute name="action">win.Add</attribute>
+          <attribute name="label" translatable="yes">_Add...</attribute>
+        </item>
+        <item>
+          <attribute name="action">win.Edit</attribute>
+          <attribute name="label">%s</attribute>
+        </item>
+        <item>
+          <attribute name="action">win.Remove</attribute>
+          <attribute name="label" translatable="yes">_Delete</attribute>
+        </item>
+        <item>
+          <attribute name="action">win.Merge</attribute>
+          <attribute name="label" translatable="yes">_Merge...</attribute>
+        </item>
+      </section>
+''' % _("action|_Edit..."),  # to use sgettext()
+        '''
+        <placeholder id='otheredit'>
+        <item>
+          <attribute name="action">win.SetActive</attribute>
+          <attribute name="label" translatable="yes">'''
+        '''Set _Home Person</attribute>
+        </item>
+        <item>
+          <attribute name="action">win.FilterEdit</attribute>
+          <attribute name="label" translatable="yes">'''
+        '''Person Filter Editor</attribute>
+        </item>
+        </placeholder>
+''',  # Following are the Toolbar items
+        '''
+    <placeholder id='CommonNavigation'>
+    <child groups='RO'>
+      <object class="GtkToolButton">
+        <property name="icon-name">go-previous</property>
+        <property name="action-name">win.Back</property>
+        <property name="tooltip_text" translatable="yes">'''
+        '''Go to the previous object in the history</property>
+        <property name="label" translatable="yes">_Back</property>
+        <property name="use-underline">True</property>
+      </object>
+      <packing>
+        <property name="homogeneous">False</property>
+      </packing>
+    </child>
+    <child groups='RO'>
+      <object class="GtkToolButton">
+        <property name="icon-name">go-next</property>
+        <property name="action-name">win.Forward</property>
+        <property name="tooltip_text" translatable="yes">'''
+        '''Go to the next object in the history</property>
+        <property name="label" translatable="yes">_Forward</property>
+        <property name="use-underline">True</property>
+      </object>
+      <packing>
+        <property name="homogeneous">False</property>
+      </packing>
+    </child>
+    <child groups='RO'>
+      <object class="GtkToolButton">
+        <property name="icon-name">go-home</property>
+        <property name="action-name">win.HomePerson</property>
+        <property name="tooltip_text" translatable="yes">'''
+        '''Go to the default person</property>
+        <property name="label" translatable="yes">_Home</property>
+        <property name="use-underline">True</property>
+      </object>
+      <packing>
+        <property name="homogeneous">False</property>
+      </packing>
+    </child>
+    </placeholder>
+''',
+        '''
+    <placeholder id='BarCommonEdit'>
+    <child groups='RW'>
+      <object class="GtkToolButton">
+        <property name="icon-name">list-add</property>
+        <property name="action-name">win.Add</property>
+        <property name="tooltip_text">%s</property>
+        <property name="label" translatable="yes">_Add...</property>
+        <property name="use-underline">True</property>
+      </object>
+      <packing>
+        <property name="homogeneous">False</property>
+      </packing>
+    </child>
+    <child groups='RW'>
+      <object class="GtkToolButton">
+        <property name="icon-name">gtk-edit</property>
+        <property name="action-name">win.Edit</property>
+        <property name="tooltip_text">%s</property>
+        <property name="label" translatable="yes">Edit...</property>
+        <property name="use-underline">True</property>
+      </object>
+      <packing>
+        <property name="homogeneous">False</property>
+      </packing>
+    </child>
+    <child groups='RW'>
+      <object class="GtkToolButton">
+        <property name="icon-name">list-remove</property>
+        <property name="action-name">win.Remove</property>
+        <property name="tooltip_text">%s</property>
+        <property name="label" translatable="yes">_Delete</property>
+        <property name="use-underline">True</property>
+      </object>
+      <packing>
+        <property name="homogeneous">False</property>
+      </packing>
+    </child>
+    <child groups='RW'>
+      <object class="GtkToolButton">
+        <property name="icon-name">gramps-merge</property>
+        <property name="action-name">win.Merge</property>
+        <property name="tooltip_text">%s</property>
+        <property name="label" translatable="yes">_Merge...</property>
+        <property name="use-underline">True</property>
+      </object>
+      <packing>
+        <property name="homogeneous">False</property>
+      </packing>
+    </child>
+    </placeholder>
+''' % (ADD_MSG, EDIT_MSG, DEL_MSG, MERGE_MSG),
+        '''
+    <menu id="Popup">
+      <section>
+        <item>
+          <attribute name="action">win.Back</attribute>
+          <attribute name="label" translatable="yes">_Back</attribute>
+        </item>
+        <item>
+          <attribute name="action">win.Forward</attribute>
+          <attribute name="label" translatable="yes">Forward</attribute>
+        </item>
+        <item>
+          <attribute name="action">win.HomePerson</attribute>
+          <attribute name="label" translatable="yes">_Home</attribute>
+        </item>
+        <item>
+          <attribute name="action">win.SetActive</attribute>
+          <attribute name="label" translatable="yes">'''
+        '''Set _Home Person</attribute>
+        </item>
+      </section>
+      <section id="PopUpTree">
+      </section>
+      <section>
+        <item>
+          <attribute name="action">win.Add</attribute>
+          <attribute name="label" translatable="yes">_Add...</attribute>
+        </item>
+        <item>
+          <attribute name="action">win.Edit</attribute>
+          <attribute name="label">%s</attribute>
+        </item>
+        <item>
+          <attribute name="action">win.Remove</attribute>
+          <attribute name="label" translatable="yes">_Delete</attribute>
+        </item>
+        <item>
+          <attribute name="action">win.Merge</attribute>
+          <attribute name="label" translatable="yes">_Merge...</attribute>
+        </item>
+      </section>
+      <section>
+        <placeholder id='QuickReport'>
+        </placeholder>
+        <placeholder id='WebConnect'>
+        </placeholder>
+      </section>
+    </menu>
+    ''' % _('action|_Edit...')  # to use sgettext()
+    ]
 
     def get_handle_from_gramps_id(self, gid):
         """
-        Return the handle of the person having the given Gramps ID. 
+        Return the handle of the person having the given Gramps ID.
         """
         obj = self.dbstate.db.get_person_from_gramps_id(gid)
         if obj:
@@ -257,7 +413,7 @@ class BasePersonView(ListView):
         else:
             return None
 
-    def add(self, obj):
+    def add(self, *obj):
         """
         Add a new person to the database.
         """
@@ -265,13 +421,13 @@ class BasePersonView(ListView):
         #the editor requires a surname
         person.primary_name.add_surname(Surname())
         person.primary_name.set_primary_surname(0)
-        
+
         try:
             EditPerson(self.dbstate, self.uistate, [], person)
         except WindowActiveError:
             pass
- 
-    def edit(self, obj):
+
+    def edit(self, *obj):
         """
         Edit an existing person in the database.
         """
@@ -282,7 +438,7 @@ class BasePersonView(ListView):
             except WindowActiveError:
                 pass
 
-    def remove(self, obj):
+    def remove(self, *obj):
         """
         Remove a person from the database.
         """
@@ -292,24 +448,24 @@ class BasePersonView(ListView):
             msg1 = self._message1_format(person)
             msg2 = self._message2_format(person)
             msg2 = "%s %s" % (msg2, data_recover_msg)
-            # This gets person to delete deom self.active_person:
-            QuestionDialog(msg1, 
-                           msg2, 
-                           _('_Delete Person'), 
+            # This gets person to delete self.active_person:
+            QuestionDialog(msg1,
+                           msg2,
+                           _('_Delete Person'),
                            self.delete_person_response,
                            parent=self.uistate.window)
         else:
             # Ask to delete; option to cancel, delete rest
             # This gets person to delete from parameter
             MultiSelectDialog(self._message1_format,
-                              self._message2_format, 
+                              self._message2_format,
                               handles,
                               self._lookup_person,
                               yes_func=self.delete_person_response,
-                              parent=self.uistate.window) # Yes
+                              parent=self.uistate.window)
 
     def _message1_format(self, person):
-        return _('Delete %s?') % (name_displayer.display(person) + 
+        return _('Delete %s?') % (name_displayer.display(person) +
                                   (" [%s]" % person.gramps_id))
 
     def _message2_format(self, person):
@@ -333,23 +489,29 @@ class BasePersonView(ListView):
 
         # create the transaction
         with DbTxn('', self.dbstate.db) as trans:
-        
+
             # create name to save
             person = self.active_person
             active_name = _("Delete Person (%s)") % name_displayer.display(person)
 
             # delete the person from the database
-            # Above will emit person-delete, which removes the person via 
+            # Above will emit person-delete, which removes the person via
             # callback to the model, so row delete is signaled
             self.dbstate.db.delete_person_from_database(person, trans)
             trans.set_description(active_name)
 
         self.uistate.set_busy_cursor(False)
 
+    def remove_object_from_handle(self, handle):
+        """
+        The remove_selected_objects method is not called in this view.
+        """
+        pass
+
     def define_actions(self):
         """
         Required define_actions function for PageView. Builds the action
-        group information required. We extend beyond the normal here, 
+        group information required. We extend beyond the normal here,
         since we want to have more than one action group for the PersonView.
         Most PageViews really won't care about this.
 
@@ -362,67 +524,20 @@ class BasePersonView(ListView):
 
         ListView.define_actions(self)
 
-        self.all_action = ActionGroup(name=self.title + "/PersonAll")
-        self.edit_action = ActionGroup(name=self.title + "/PersonEdit")
-
-        self.all_action.add_actions([
-                ('FilterEdit', None, _('Person Filter Editor'), None, None,
-                self.filter_editor),
-                ('Edit', 'gtk-edit', _("action|_Edit..."),
-                "<PRIMARY>Return", self.EDIT_MSG, self.edit), 
-                ('QuickReport', None, _("Quick View"), None, None, None), 
-                ('WebConnect', None, _("Web Connection"), None, None, None), 
-                ])
-
-
-        self.edit_action.add_actions(
-            [
-                ('Add', 'list-add', _("_Add..."), "<PRIMARY>Insert",
-                 self.ADD_MSG, self.add),
-                ('Remove', 'list-remove', _("_Remove"), "<PRIMARY>Delete",
-                 self.DEL_MSG, self.remove),
-                ('Merge', 'gramps-merge', _('_Merge...'), None,
-                 self.MERGE_MSG, self.merge),
-                ('ExportTab', None, _('Export View...'), None, None,
-                 self.export),
-                ])
-
-        self._add_action_group(self.edit_action)
-        self._add_action_group(self.all_action)
-
-    def enable_action_group(self, obj):
-        """
-        Turns on the visibility of the View's action group.
-        """
-        ListView.enable_action_group(self, obj)
-        self.all_action.set_visible(True)
-        self.edit_action.set_visible(True)
-        self.edit_action.set_sensitive(not self.dbstate.db.readonly)
-        
-    def disable_action_group(self):
-        """
-        Turns off the visibility of the View's action group.
-        """
-        ListView.disable_action_group(self)
-
-        self.all_action.set_visible(False)
-        self.edit_action.set_visible(False)
-
-    def merge(self, obj):
+    def merge(self, *obj):
         """
         Merge the selected people.
         """
         mlist = self.selected_handles()
 
         if len(mlist) != 2:
-            ErrorDialog(
-        _("Cannot merge people"), 
-        _("Exactly two people must be selected to perform a merge. "
-          "A second person can be selected by holding down the "
-          "control key while clicking on the desired person."),
-        parent=self.uistate.window)
+            ErrorDialog(_("Cannot merge people"),
+                        _("Exactly two people must be selected to perform "
+                        "a merge. A second person can be selected by "
+                        "holding down the control key while clicking on "
+                        "the desired person."), parent=self.uistate.window)
         else:
-            MergePerson(self.dbstate, self.uistate, mlist[0], mlist[1])
+            MergePerson(self.dbstate, self.uistate, [], mlist[0], mlist[1])
 
     def tag_updated(self, handle_list):
         """
@@ -443,7 +558,7 @@ class BasePersonView(ListView):
         person = self.dbstate.db.get_person_from_handle(person_handle)
         person.add_tag(tag_handle)
         self.dbstate.db.commit_person(person, transaction)
-        
+
     def get_default_gramplets(self):
         """
         Define the default gramplets for the sidebar and bottombar.
