@@ -729,7 +729,8 @@ class ProgenParser(UpdateCallback):
 
         # provide feedback about import progress (GUI / TXT)
         if self.uistate:
-            self.progress = ProgressMeter(_("Import from Pro-Gen"), '')
+            self.progress = ProgressMeter(_("Import from Pro-Gen"), '',
+                                          parent=self.uistate.window)
         else:
             UpdateCallback.__init__(self, user.callback)
 
@@ -1331,9 +1332,10 @@ class ProgenParser(UpdateCallback):
                     if attr:
                         person.add_attribute(attr)
                 else:
-                    self.__add_name(person, citation.handle, NameType.AKA,
-                        ' '.join(alias_text[0:-1]),
-                        '', alias_text[-1].split(), '')
+                    self.__add_name(
+                        person, citation.handle if citation else None,
+                        NameType.AKA, ' '.join(alias_text[0:-1]),
+                        '', alias_text[-1], '')
 
             # process F09 Person Code
             refn_code = recflds[person_ix[9]]   # F09: INDI REFN/INDI CODE
