@@ -12,7 +12,7 @@
 # Copyright (C) 2008-2011  Rob G. Healey <robhealey1@gmail.com>
 # Copyright (C) 2010       Doug Blank <doug.blank@gmail.com>
 # Copyright (C) 2010       Jakim Friant
-# Copyright (C) 2010-2017  Serge Noiraud
+# Copyright (C) 2010-      Serge Noiraud
 # Copyright (C) 2011       Tim G L Lyons
 # Copyright (C) 2013       Benny Malengier
 # Copyright (C) 2016       Allen Crider
@@ -64,7 +64,7 @@ from gramps.plugins.webreport.common import (get_first_letters, first_letter,
                                              primary_difference, _KEYPLACE,
                                              get_index_letter, FULLCLEAR,
                                              MARKER_PATH, OSM_MARKERS, MARKERS,
-                                             html_escape)
+                                             html_escape, sort_places)
 
 _ = glocale.translation.sgettext
 LOG = logging.getLogger(".NarrativeWeb")
@@ -100,6 +100,9 @@ class PlacePages(BasePage):
         self.familymappages = None
         self.googlemapkey = None
 
+        # Place needs to display coordinates?
+        self.display_coordinates = report.options["coordinates"]
+
     def display_pages(self, title):
         """
         Generate and output the pages under the Place tab, namely the place
@@ -135,13 +138,14 @@ class PlacePages(BasePage):
         BasePage.__init__(self, report, title)
 
         output_file, sio = self.report.create_file("places")
-        placelistpage, head, body = self.write_header(self._("Places"))
+        result = self.write_header(self._("Places"))
+        placelistpage, dummy_head, dummy_body, outerwrapper = result
         ldatec = 0
         prev_letter = " "
 
         # begin places division
         with Html("div", class_="content", id="Places") as placelist:
-            body += placelist
+            outerwrapper += placelist
 
             # place list page message
             msg = self._("This page contains an index of all the places in the "
@@ -169,33 +173,38 @@ class PlacePages(BasePage):
                 trow = Html("tr")
                 thead += trow
 
-                trow.extend(
-                    Html("th", label, class_=colclass, inline=True)
-                    for (label, colclass) in [
-                        [self._("Letter"), "ColumnLetter"],
-                        [self._("Place Name | Name"), "ColumnName"],
-                        [self._("State/ Province"), "ColumnState"],
-                        [self._("Country"), "ColumnCountry"],
-                        [self._("Latitude"), "ColumnLatitude"],
-                        [self._("Longitude"), "ColumnLongitude"]
-                    ]
-                )
+                if self.display_coordinates:
+                    trow.extend(
+                        Html("th", label, class_=colclass, inline=True)
+                        for (label, colclass) in [
+                            [self._("Letter"), "ColumnLetter"],
+                            [self._("Place Name | Name"), "ColumnName"],
+                            [self._("State/ Province"), "ColumnState"],
+                            [self._("Country"), "ColumnCountry"],
+                            [self._("Latitude"), "ColumnLatitude"],
+                            [self._("Longitude"), "ColumnLongitude"]
+                        ]
+                    )
+                else:
+                    trow.extend(
+                        Html("th", label, class_=colclass, inline=True)
+                        for (label, colclass) in [
+                            [self._("Letter"), "ColumnLetter"],
+                            [self._("Place Name | Name"), "ColumnName"],
+                            [self._("State/ Province"), "ColumnState"],
+                            [self._("Country"), "ColumnCountry"]
+                        ]
+                    )
 
-                # bug 9495 : incomplete display of place hierarchy labels
-                def sort_by_place_name(obj):
-                    """ sort by lower case place name. """
-                    name = self.report.obj_dict[Place][obj][1]
-                    return name.lower()
-
-                handle_list = sorted(place_handles,
-                                     key=lambda x: sort_by_place_name(x))
+                handle_list = sort_places(self.r_db, place_handles,
+                                          self.rlocale)
                 first = True
 
                 # begin table body
                 tbody = Html("tbody")
                 table += tbody
 
-                for place_handle in handle_list:
+                for (dummy_pname, place_handle) in handle_list:
                     place = self.r_db.get_place_from_handle(place_handle)
                     if place:
                         if place.get_change_time() > ldatec:
@@ -244,26 +253,27 @@ class PlacePages(BasePage):
                             ]
                         )
 
-                        tcell1 = Html("td", class_="ColumnLatitude",
-                                      inline=True)
-                        tcell2 = Html("td", class_="ColumnLongitude",
-                                      inline=True)
-                        trow += (tcell1, tcell2)
+                        if self.display_coordinates:
+                            tcell1 = Html("td", class_="ColumnLatitude",
+                                          inline=True)
+                            tcell2 = Html("td", class_="ColumnLongitude",
+                                          inline=True)
+                            trow += (tcell1, tcell2)
 
-                        if place.lat and place.long:
-                            latitude, longitude = conv_lat_lon(place.lat,
-                                                               place.long,
-                                                               "DEG")
-                            tcell1 += latitude
-                            tcell2 += longitude
-                        else:
-                            tcell1 += '&nbsp;'
-                            tcell2 += '&nbsp;'
+                            if place.lat and place.long:
+                                latitude, longitude = conv_lat_lon(place.lat,
+                                                                   place.long,
+                                                                   "DEG")
+                                tcell1 += latitude
+                                tcell2 += longitude
+                            else:
+                                tcell1 += '&nbsp;'
+                                tcell2 += '&nbsp;'
 
         # add clearline for proper styling
         # add footer section
         footer = self.write_footer(ldatec)
-        body += (FULLCLEAR, footer)
+        outerwrapper += (FULLCLEAR, footer)
 
         # send page out for processing
         # and close the file
@@ -280,7 +290,7 @@ class PlacePages(BasePage):
         """
         place = report.database.get_place_from_handle(place_handle)
         if not place:
-            return None
+            return
         BasePage.__init__(self, report, title, place.get_gramps_id())
         self.bibli = Bibliography()
         place_name = self.report.obj_dict[Place][place_handle][1]
@@ -289,7 +299,7 @@ class PlacePages(BasePage):
         output_file, sio = self.report.create_file(place_handle, "plc")
         self.uplink = True
         self.page_title = place_name
-        placepage, head, body = self.write_header(_("Places"))
+        placepage, head, body, outerwrapper = self.write_header(_("Places"))
 
         self.placemappages = self.report.options['placemappages']
         self.mapservice = self.report.options['mapservice']
@@ -297,7 +307,7 @@ class PlacePages(BasePage):
 
         # begin PlaceDetail Division
         with Html("div", class_="content", id="PlaceDetail") as placedetail:
-            body += placedetail
+            outerwrapper += placedetail
 
             if self.create_media:
                 media_list = place.get_media_list()
@@ -374,12 +384,10 @@ class PlacePages(BasePage):
                                    "jquery.min.js")
                         head += Html("script", type="text/javascript",
                                      src=src_js, inline=True)
-                        src_js = self.secure_mode
-                        src_js += "openlayers.org/en/v3.17.1/build/ol.js"
+                        src_js = "https://openlayers.org/en/latest/build/ol.js"
                         head += Html("script", type="text/javascript",
                                      src=src_js, inline=True)
-                        url = self.secure_mode
-                        url += "openlayers.org/en/v3.17.1/css/ol.css"
+                        url = "https://openlayers.org/en/latest/css/ol.css"
                         head += Html("link", href=url, type="text/javascript",
                                      rel="stylesheet")
                         src_js = self.secure_mode
@@ -445,7 +453,7 @@ class PlacePages(BasePage):
         # add clearline for proper styling
         # add footer section
         footer = self.write_footer(ldatec)
-        body += (FULLCLEAR, footer)
+        outerwrapper += (FULLCLEAR, footer)
 
         # send page out for processing
         # and close the file
